import {
  BigIntPr,
  Byte,
  Bytes,
  Int,
  Short,
  Stringpr,
} from '../serializable/primitives';
import { concatBytes } from '../utils/buffer';
import { bytesForInt } from './utils/bytesFor';

export const intBytes = () => new Uint8Array([0x00, 0x00, 0x00, 0x0d]);

export const int = () => new Int(13);

export const intsBytes = () =>
  concatBytes(bytesForInt(3), bytesForInt(3), bytesForInt(2), bytesForInt(1));

export const ints = () => [new Int(3), new Int(2), new Int(1)];

export const byteByte = () => new Uint8Array([0x01]);

export const byte = () => new Byte(new Uint8Array([0x01]));

export const bytesBytes = () =>
  concatBytes(bytesForInt(2), new Uint8Array([0x01, 0x02]));

export const bytes = () => new Bytes(new Uint8Array([0x01, 0x02]));

export const bigIntPrBytes = () =>
  new Uint8Array([0x00, 0x00, 0x00, 0x00, 0x00, 0x1e, 0x84, 0x80]);

export const bigIntPr = () => new BigIntPr(2000000n);

export const stringPr = () => new Stringpr('Avax');

export const stringPrBytes = () =>
  new Uint8Array([0x00, 0x04, 0x41, 0x76, 0x61, 0x78]);

export const shortBytes = () => new Uint8Array([0x00, 0x0d]);

export const short = () => new Short(13);

export const blsPublicKeyBytes = () =>
  new Uint8Array([
    0x85, 0x02, 0x5b, 0xca, 0x6a, 0x30, 0x2d, 0xc6, 0x13, 0x38, 0xff, 0x49,
    0xc8, 0xba, 0xa5, 0x72, 0xde, 0xd3, 0xe8, 0x6f, 0x37, 0x59, 0x30, 0x4c,
    0x7f, 0x61, 0x8a, 0x2a, 0x25, 0x93, 0xc1, 0x87, 0xe0, 0x80, 0xa3, 0xcf,
    0xde, 0xc9, 0x50, 0x40, 0x30, 0x9a, 0xd1, 0xf1, 0x58, 0x95, 0x30, 0x67,
  ]);

export const blsSignatureBytes = () =>
  new Uint8Array([
    0x8b, 0x1d, 0x61, 0x33, 0xd1, 0x7e, 0x34, 0x83, 0x22, 0x0a, 0xd9, 0x60,
    0xb6, 0xfd, 0xe1, 0x1e, 0x4e, 0x12, 0x14, 0xa8, 0xce, 0x21, 0xef, 0x61,
    0x62, 0x27, 0xe5, 0xd5, 0xee, 0xf0, 0x70, 0xd7, 0x50, 0x0e, 0x6f, 0x7d,
    0x44, 0x52, 0xc5, 0xa7, 0x60, 0x62, 0x0c, 0xc0, 0x67, 0x95, 0xcb, 0xe2,
    0x18, 0xe0, 0x72, 0xeb, 0xa7, 0x6d, 0x94, 0x78, 0x8d, 0x9d, 0x01, 0x17,
    0x6c, 0xe4, 0xec, 0xad, 0xfb, 0x96, 0xb4, 0x7f, 0x94, 0x22, 0x81, 0x89,
    0x4d, 0xdf, 0xad, 0xd1, 0xc1, 0x74, 0x3f, 0x7f, 0x54, 0x9f, 0x1d, 0x07,
    0xd5, 0x9d, 0x55, 0x65, 0x59, 0x27, 0xf7, 0x2b, 0xc6, 0xbf, 0x7c, 0x12,
  ]);
